#!/usr/bin/perl

#--------------------------------------------------------------
# debug_message ()
#
# This sub-function prints the string passed to it.
# This sub-function should be used only for debug messages.
#
# @param the messages string to be printed
#
#--------------------------------------------------------------
sub debug_message ($)
{
    if ($DEBUG eq $TRUE)
    {
	display_message (@_);
    }
}


#--------------------------------------------------------------
# display_message ()
#
# This sub-function prints the string passed to it.
# This sub-function should be used to convey information to
# users
#
# @param the messages string to be printed
#
#--------------------------------------------------------------
sub display_message ($)
{
    local ($str) = @_;
    print "$str";
}


#--------------------------------------------------------------
# usage ()
#
# Prints the Usage description
#--------------------------------------------------------------
sub usage ()
{
    debug_message ("Enter usage ()...\n");

    local ($USAGE) = "";
    local ($USAGE_SYNOPSIS1) = "";
    local ($USAGE_SYNOPSIS2) = "";
    local ($USAGE_OPTIONS) = "";
 
    $USAGE_SYNOPSIS1 = "	$0   [-h | --help] [-V | --version] [-v | --verbose] \n";
    $USAGE_SYNOPSIS2 = "	     	    [-N | --groups <group,group,group,...>] \n 		    [-n | --hosts  <host_name,host_name,host_name,...>]\n		    [command_specification]\n\n";	

    $USAGE_OPTIONS = "	-h, --help : Output usage message. \n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	-V, --version : Output version information.\n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	-v, --verbose : Run in debug mode. \n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	-N, --groups <group,group,group,...> : Overrides the groups and\n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	clears hosts fields.\n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	-n, --hosts  <host_name,host_name,host_name,...> : Overrides the Host names\n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	and clears groups field.\n";

    $USAGE = "Usage:\n";
    $USAGE = ($USAGE) . ($USAGE_SYNOPSIS1) . ($USAGE_SYNOPSIS2); 
    $USAGE = ($USAGE) . ($USAGE_OPTIONS);
    # Use the message catalog
    #`dspmsg dcem.cat 10 \"$USAGE\"`

    display_message ($USAGE);

    debug_message ("Leave usage ().\n");
    exit (1);

}


#--------------------------------------------------------------
# run_dcem ()
#
# Executes the "djava" command with the appropriate arguments
#--------------------------------------------------------------
sub run_dcem ()
{
    debug_message ("Enter run_dcem ()...\n");

    local ($CMD) = ($DJAVA);
    
    if ($DEBUG eq $TRUE)
    {
	$CMD = ($CMD) . (" ") . ("-debug");
    }

    $CMD = ($CMD) . (" ") . ($JAVA_CLASS_NAME) . (" ") . ($DISTRIBUTED_CMD);

    debug_message ("Executing: $CMD\n");
    select (STDOUT);
    $| = 1;
    select (STDERR);
    $| = 1;
    exec ("$CMD");

    debug_message ("Leave run_dcem ().\n");
}


#--------------------------------------------------------------
# process_arguments ()
#
# Process the command line arguments passed to this script
#--------------------------------------------------------------
sub process_arguments ()
{
    debug_message ("Enter process_arguments ()...\n");

    local ($arg)	= "";
    local ($count)	= 0;
    local ($groupArgFound) = $FALSE;
    local ($hostArgFound) = $FALSE;
    local ($commandArgFound) = $FALSE;
    local ($versionArgFound) = $FALSE;
    local ($debugArgFound)   = $FALSE;
    
    $count = @ARGV;
    
    while ($count > 0)
    {
	debug_message ("Argument = $ARGV[0]\n");
        
        if (($ARGV[0] !~ /^-/) && ($commandArgFound eq $FALSE))
	{
	    $commandArgFound=$TRUE;
	    $DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
	}
        elsif ((($ARGV [$0] eq "--groups") || ($ARGV [$0] eq "-N")) && ($groupArgFound eq $FALSE))
        {
            $groupArgFound = $TRUE;
            $DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
	    shift (@ARGV); # discard the first argument in the array
            $count--;

            if (($count < 1) || (substr($ARGV[0],0,1) eq "-"))  # there must be group list argument
            {
                usage ();
            }
            else
	    {
		debug_message ("GROUPS = $ARGV[0]\n");
		$DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
	    }
        }
	elsif ((($ARGV [$0] eq "--hosts")  || ($ARGV [$0] eq "-n")) && ($hostArgFound eq $FALSE))
        {
            $hostArgFound = $TRUE;
            $DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
	    shift (@ARGV); # discard the first argument in the array
            $count--;

            if (($count < 1) || (substr($ARGV[0],0,1) eq "-"))  # there must be host list argument
            {
                usage ();
            }
            else
	    {
		debug_message ("HOSTS = $ARGV[0]\n");
		$DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
	    }
        }
	elsif ((($ARGV [$0] eq "--version") || ($ARGV [$0] eq "-V")) && ($versionArgFound eq $FALSE))
	{
		$versionArgFound=$TRUE;
		display_message($VERSION);
		
	}
	elsif (($ARGV [$0] eq "--help") || ($ARGV [$0] eq "-h")) 
	{
		usage();
	}
	else # other arguments
	{
	    usage ();
	}
	
	shift (@ARGV); # discard the first argument in the array
	$count--;
    }

    run_dcem ();

    debug_message ("Leave process_arguments ().\n");
}


#--------------------------------------------------------------
# main ()
#
#--------------------------------------------------------------
sub main ()
{
    debug_message ("Enter main ()...\n");

    process_arguments ();

    debug_message ("Leave main ().\n");
}


#--------------------------------------------------------------
# START of the perl script.
#
#--------------------------------------------------------------
$|				= 1;

$TRUE				= 1;
$FALSE				= 0;
$DEBUG				= $FALSE;

$DJAVA				= "/opt/csm/dcem/bin/djava";
$DISTRIBUTED_CMD		= "";
$VERSION			= "DCEM version 1.1.0.0\n";
$JAVA_CLASS_NAME		= "com.ibm.unix.distrib.framework.ui.DUICommandAppDialog";

#--------------------------------------------------------------
# START of the main function
#
#--------------------------------------------------------------
# Look for the debug option first
if (($ARGV[0] eq "--verbose") || ($ARGV[0] eq "-v")) 
{
    $DEBUG = $TRUE;
    debug_message ("Turning DEBUG ON\n");
    shift (@ARGV); # discard the first argument, i.e. -debug
}

main ();
