#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2001,2002 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

#
#####################################################################
#                                                                   #
# Module: InstallUtils.pm                                           #
#                                                                   #
#-------------------------------------------------------------------#
#                                                                   #
# Description: This module contains Perl subroutines used           #
#              commonly across the CSM installation commands.       #
#              Detailed descriptions appear in subroutine headers.  #
#                                                                   #
# Subroutines:                                                      #
#								    #	  
#  make_node_list_file						    #
#  get_target_nodes get_common_attrs                                #
#  make_config_file create_nodemap_file start_nfs export_csminstall #
#  unexport_csminstall 						    #
#  get_lock   release_lock  backup_logfile start_logging            #
#  stop_logging  append_logging  get_management_server  get_OSName  #
#  get_DistributionVersion                                          #
#  get_CSMVersion  get_PkgArchitecture  get_pkgVersion              #
#  get_DistributionName get_uniqu_arry_elemnts                      #
#  findRpmInPath findRpm installRequiredRpms getRequiredRpms        #
#  getReqsNotSatisfied parseRequire inc_ip                          #
#  SetAllowRequest SetMode genHex verify_tftp verify_dhcp           #
#  setup_atftp testvalidconsole testvalidpowermethod chkethup       #
#  getRecentVersionRpm subnet                                       #
#                                                                   #
#####################################################################
# @(#)91   1.79.1.23   src/csm/install/InstallUtils.pm.perl, setup, csm_rori 10/2/02 15:47:07

package InstallUtils;

use strict;
use Fcntl qw(:flock);

BEGIN 
{ $::csmpm = $ENV{'CSM_PM'} ? $ENV{'CSM_PM'} : '/opt/csm/pm'; }
use lib $::csmpm;

#
# Subroutines
#

###########################################################
# make_node_list_file
#  this file is suitable for the ENV vars: 
#	DSH_LIST
#	RPOWER_LIST
#	RCONSOLE_LIST
#
#  IMPORTANT!!!: if you use this subroutine, make sure you use 
#	InstallUtils->close_delete_file($file_handle, $file_name) 
#       to erase it.
#
# input: (\@list_of_nodes)
# output
#	$file_name
# sets global var: $::NODE_LIST_FILE
###########################################################
sub make_node_list_file{
	my ($class, $ref_node_list) = @_;
	my @node_list = @$ref_node_list;
	srand(time|$$); #random number generator start
	
	my $file = "/tmp/$$";
        while(-e $file){
             $file = InstallUtils->CreateRandomName($file);
        }

	open($::NODE_LIST_FILE, ">$file") or NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', "$file");
	foreach my $node (@node_list){
		print $::NODE_LIST_FILE "$node\n";
	}
	return $file;

}
###########################################################
# close_delete_file
# This subroutine takes a file handle and file name
# First closes and then deletes the file
# input: $file_handle $file_name
# output null
###########################################################
sub close_delete_file{
	my ($class, $file_handle, $file_name) = @_;
	close $file_handle;
	
  	my $cmd = "$::RM -f $file_name";
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCMD', $cmd);
        my $rc  = system ($cmd) >> 8;
        if ($rc) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        }
}
###########################################################
# CreateRandomName
# input : prefix of name
# output: prefix with 8 random letters appended
###########################################################
sub CreateRandomName {

    my ($class, $name) = @_;

    my $nI;
    for ($nI = 0; $nI < 8; $nI++) {
       my $char = ('a'..'z', 'A'..'Z')[int(rand( 52)) + 1];
       $name .= $char;
    }
    $name;
}
###########################################################
# get_target_nodes
#
# input:
#      (\@ARGV) either from @ARGV or $::opt_n
#      ($node group string) from $::opt_N
#
#Global variables:
#	$::ALL                     	#Distribute to all nodes
#	$::MANAGED     	#Distribute to all nodes where Mode="Managed"
#       $::PRE_MANAGED	#Distribute to all nodes where Mode="PreManaged"
#
# output
#	\@DestNode  = reference to list of all target nodes
#       \@lsnode_info = reference to a list of results from lsnode command (only for the nodes in @DestNode)
#	\%DestNodeHash = reference to a hash of hostnames and attributes (foreach node in @Destnode):
#			$DestNodeHash{$hostname}{$attribute_name} = "attribute value"
#			For the list of attributes see $lsnode_attr
###########################################################
sub get_target_nodes{
    my ($class, $ref_nodelist, $nodegrp_list) = @_;
    my @nodelist = @$ref_nodelist;
    my (@DestNode, @DestGroup, %DestKeyTemp, %all_groups, @lsnode_info, @empty_array);
    
    my @real_nodelist; #split the nodes if there are commas
    foreach my $n (@nodelist){
	   if($n =~ m/,/){
		   my @tmp = split ',', $n;
		   foreach my $t (@tmp){
			   push @real_nodelist, $t;
		   }
	   }
	   else{
		   push @real_nodelist, $n;
	   }
   }
   @nodelist=@real_nodelist;
   undef @real_nodelist;
    
    my $lsnode_attr = "Hostname,ManagementServer,InstallOSName,InstallCSMVersion,InstallDistributionVersion,InstallDistributionName,InstallPkgArchitecture,Mode,InstallAdapterMacaddr,InstallAdapterType,InstallMethod,ConsoleSerialDevice";
    # These delimeters are used with the lsnode command.  A simple colon (:)
    # cannot be used since there are colons in InstallAdapterMacaddr.
    #my $delim = ':|:';		# Used on the lsnode command
    my $check_delim = ':\|:';	# Used when splitting the line apart
    
    if($::ALL || $::MANAGED || $::PRE_MANAGED ){ #distribute to all nodes of a certain set
        my $select_string;
        if($::MANAGED){
            $select_string = "Mode=\"Managed\"";
        } elsif($::PRE_MANAGED){
            $select_string = "Mode=\"PreManaged\"";
        } else{ #just managednodes
            $select_string = "";
        }
        my %opthash = (WhereStr => $select_string, SkipResolve => 1); 
	my $ref_lsnode_info = NodeUtils->listNodeAttrs( \@empty_array, $lsnode_attr, \%opthash);
	@lsnode_info = @$ref_lsnode_info;
        
        foreach my $line (@lsnode_info){
            chomp $line;
            my $hostname = (split /$check_delim/, $line)[0];
            push @DestNode, $hostname;
        }
    } 
    else{ #we are not distributing to all nodes of a certain mode        
	my %opthashref = ( NodeFile => 1, NodeGrp => $nodegrp_list);
	my $refnodes = NodeUtils->gatherNodeList(\@nodelist, \%opthashref);
	my $refrealnodes = NodeUtils->resolveAndUndup($refnodes);
	foreach my $node (@$refrealnodes){
	    push @DestNode, $node;   
	}
	if(@DestNode){
		my %opthash = ( SkipResolve => 1);
		my $ref_lsnode_info = NodeUtils->listNodeAttrs( \@DestNode, $lsnode_attr, \%opthash);
		@lsnode_info = @$ref_lsnode_info;
	}
	else{
		@lsnode_info = ();
	}
    }
    
    my  %DestNodeHash;
    foreach my $line (@lsnode_info){  #this only has node names in it from the cluster
	my @attributes = split /$check_delim/, $line;
	my $hostname = $attributes[0];
	chomp $line;
	my @attr_names = split ',', $lsnode_attr;
	my $index =0;
	foreach my $attr (@attr_names){
	    $DestNodeHash{$hostname}{$attr} = $attributes[$index];
	    $index++;
	}
    }
    
    if(@::VALID_INSTALLMETHODS){
	my %meths;
	foreach my $method (@::VALID_INSTALLMETHODS){
	    $meths{$method}=1;
	}
  	 my @methString = keys %meths;
	foreach my $target (keys %DestNodeHash){
	    if(!$meths{$DestNodeHash{$target}{'InstallMethod'}}){
		if(!($::ALL || $::MANAGED || $::PRE_MANAGED)){                  
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgINVALID_INSTALL_METH', $target, "@methString");
		}
		else{
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','W', 'EMsgINVALID_INSTALL_METH', $target, "@methString");
			delete $DestNodeHash{$target};
		    }
	    }
	}
    }

    my @tmpDestNode = keys %DestNodeHash;
    my @return_array = (\@tmpDestNode, \@lsnode_info, \%DestNodeHash);
    return @return_array;
}

# get_common_attrs
#
# Get the common attributes across all the nodes.
#  InstallDistributionName
#  InstallDistributionVersion
#  InstallPkgArchitecture
#  InstallCSMVersion
#
# input:
#      none
# global varialbles:
#      %::NODEHASH - List of nodes to check
# output:
#      The following variables in an array:
#      ($distro_name, $distro_version, $arch, $csm_version);
#
# Exits with an error if one of the attributes is not the same on 
# all the nodes, 
#
sub get_common_attrs
{
    my ($hostname, $distro_name, $distro_version, $arch, $csm_version);
    my ($errors) = 0;
    my ($InstallDistributionName);
    my ($InstallDistributionVersion);
    my ($InstallPkgArchitecture);
    my ($InstallCSMVersion);

    foreach $hostname (keys %::NODEHASH)
    {
	$InstallDistributionName = 
	    $::NODEHASH{$hostname}{'InstallDistributionName'};
	$InstallDistributionVersion = 
	    $::NODEHASH{$hostname}{'InstallDistributionVersion'};
	$InstallPkgArchitecture = 
	    $::NODEHASH{$hostname}{'InstallPkgArchitecture'};
	$InstallCSMVersion = 
	    $::NODEHASH{$hostname}{'InstallCSMVersion'};

	if (! $distro_name)
	{
	    $distro_name = $InstallDistributionName;
	}
	if (! $distro_version)
	{
	    $distro_version = $InstallDistributionVersion;
	}
	if (! $arch)
	{
	    $arch = $InstallPkgArchitecture;
	}
	if (! $csm_version)
	{
	    $csm_version = $InstallCSMVersion;
	}

	if ("$distro_name" and 
	    ($InstallDistributionName ne $distro_name))
	{
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgDIFFERENT_ATTR', 'InstallDistributionName');
	    $errors++;
	}

	if ("$distro_version" and 
	    ($InstallDistributionVersion ne $distro_version))
	{
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgDIFFERENT_ATTR', 'InstallDistributionVersion');
	    $errors++;
	}

	if ("$arch" and 
	    ($InstallPkgArchitecture ne $arch))
	{
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgDIFFERENT_ATTR', 'InstallPkgArchitecture');
	    $errors++;
	}

	if ("$csm_version" and 
	    ($InstallCSMVersion ne $csm_version))
	{
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgDIFFERENT_ATTR', 'InstallCSMVersion');
	    $errors++;
	}
    }

    if (! $distro_name) {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgNO_ATTR', 'InstallDistributionName');
	$errors++;
    }
    if (! $distro_version) {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgNO_ATTR', 'InstallDistributionVersion');
	$errors++;
    }
    if (! $arch) {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgNO_ATTR', 'InstallPkgArchitecture');
	$errors++;
    }
    if (! $csm_version) {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgNO_ATTR', 'InstallCSMVersion');
	$errors++;
    }


    exit $errors if ($errors);

    my @return_array = ($distro_name, $distro_version, $arch, $csm_version);
    return (@return_array);
}

#-------------------------------------------------------------------------#
#                                                                         #
#  SetAllowRequest     Make sure the management server can update the     #
#                      node information. 				  #	
#		       	This should be set to 3 by installnode		  #
#		 	and by updatenode if the Mode is not managed.	  #
#			Otherwise it should be set to 2 by updatenode.    #
#			during updatenode exit, it should be set to 0.    #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub SetAllowRequest {

    my ($class, $ref_DestNodeHash, $value) = @_;
    my %DestNodeHash = %$ref_DestNodeHash;
    if(!$value){
	    $value= "0 ";
    }
    if(keys %DestNodeHash){
	    # DestNodeHash is a hash containing attribute values for 
	    # each node that was provided on the command line.
	    my ( $cmd, $rc);

	    # check each node on our list and set the AllowManageRequest 
	    # attribute if needed.
	    my @nodelist = (keys %DestNodeHash);
	    
	    my $nodelist_file = 
		InstallUtils->make_node_list_file(\@nodelist);
	    $cmd = "$::CHNODE -f $nodelist_file AllowManageRequest=$value";
	    $rc = system("$cmd");
	    if ($rc >> 8){
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', "$cmd", $rc);
	    }
	    InstallUtils->close_delete_file($::NODE_LIST_FILE, $nodelist_file);
	    return $::OK;
	    
    }
    else{
	    return 1; #a value and nodes must be specified.
    }
}

# Set a node's Mode attribute to PreManaged, Installing or Managed.
sub SetMode
{
    my ($class, $node, $mode) = @_;
    my ($cmd, $rc, $fstatus);

    if (($mode ne "PreManaged") and 
	($mode ne "Installing") and 
	($mode ne "Managed"))
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgINVALID_MODE', $mode, "PreManaged, Installing, Managed");
    }

    $cmd = "$::CHNODE $node Mode=$mode";
    $rc = system($cmd) >> 8;
    if ($rc != 0)
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', $::CHNODE, $rc);
	$fstatus = $rc;
    }

    return $fstatus;
}

#-------------------------------------------------------------------------#
#                                                                         #
#  make_config_file  - Create configuration information files for each    #
#                      node provided in the list.                         #
#                                                                         #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub make_config_file {

    my ($class, $ref_nl, $ref_nodeinfo, $ref_DestNodeHash) = @_;
	my @nl = @$ref_nl;
	my @nodeinfo = @$ref_nodeinfo;

    #  DestNodeHash is a hash containing attribute values for each node
    #    that was provided on the command line.
	my %DestNodeHash = %$ref_DestNodeHash;

    my ($hname, $rc);
    my $csmcfgdir = "/csminstall/csm/config";
    my $cmd;

    #
    #  Check and/or create the /csminstall/csm/config directory
    #
    if (! -d $csmcfgdir) {
        $cmd = "$::MKDIR -m 755 -p $csmcfgdir";
        $rc = system("$cmd");
        if ($rc >> 8) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_CreateDir', $csmcfgdir);
            return($::NOK);
        }
    }

    #
    # create the config_info file
    #

    #  go through each node in the list
    #   (the keys are the hostnames as returned from tryHost() !!)
    foreach $hname (keys %DestNodeHash) { 
        
        # remove the old config file if any
        if ( -e "$csmcfgdir/$hname.config_info") {
            $cmd = "$::RM $csmcfgdir/$hname.config_info";
            $rc = system("$cmd");
            if ($rc >> 8){
                my $fn = "$csmcfgdir/$hname.config_info";
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_RM', "$fn");
                 # try to continue ??return $::NOK;
            }
        }

        # create the new file
        unless (open(CONFIG_OUT,">$csmcfgdir/$hname.config_info")) {
            my $fn = "$csmcfgdir/$hname.config_info";
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', "$fn");
            return $::NOK;
        }

        # add the node configuration information to the file
        if ( defined($hname) ) {
            print CONFIG_OUT "Hostname=$hname\n";
        }
        if ( defined($DestNodeHash{$hname}{'ManagementServer'}) ) {
            print CONFIG_OUT "ManagementServer=$DestNodeHash{$hname}{'ManagementServer'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'InstallOSName'} ) ) {
            print CONFIG_OUT "InstallOSName=$DestNodeHash{$hname}{'InstallOSName'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'InstallCSMVersion'}) ) {
            print CONFIG_OUT "InstallCSMVersion=$DestNodeHash{$hname}{'InstallCSMVersion'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'InstallDistributionVersion'}) ) {
            print CONFIG_OUT "InstallDistributionVersion=$DestNodeHash{$hname}{'InstallDistributionVersion'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'InstallDistributionName'} ) ) {
            print CONFIG_OUT "InstallDistributionName=$DestNodeHash{$hname}{'InstallDistributionName'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'InstallPkgArchitecture'}) ) {
            print CONFIG_OUT "InstallPkgArchitecture=$DestNodeHash{$hname}{'InstallPkgArchitecture'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'Mode'}) ) {
	    print CONFIG_OUT "Mode=$DestNodeHash{$hname}{'Mode'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'ConsoleSerialDevice'}) ) {
	    print CONFIG_OUT "ConsoleSerialDevice=$DestNodeHash{$hname}{'ConsoleSerialDevice'}\n";
        }

        close(INSTALL_OUT);
        
        $cmd = "$::CHMOD 644 $csmcfgdir/$hname.config_info";
        $rc = system("$cmd");
        if ($rc >> 8){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', $::CHMOD, $rc);
            # try to continue
        }
    }  # go do the next node

    return $::OK;
}

#-------------------------------------------------------------------------#
#    create_nodemap_file                                                  #
#                                                                         #
# Create /csminstall/csm/config/nodemap file that contains a mapping of   #
# ManagedNode hostnames as defined in the CSM database, and node          #
# hostnames as returned by the hostname command.                          #
#                                                                         #
# File format:                                                            #
#      csm_hostname<space>node_hostname                                   #
#                                                                         #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub create_nodemap_file {
    my ($class, $ref_nodelist, $ref_nodeinfo, $ref_DestNodeHash) = @_;
    my @nodelist = @$ref_nodelist;
    my @nodeinfo = @$ref_nodeinfo;
    #  DestNodeHash is a hash containing attribute values for each node
    #    that was provided on the command line.
    my %DestNodeHash = %$ref_DestNodeHash;
    %main::dsh_failed = ();
    my ($cmd, @dshout, $rc);
    my ($csm_nodename, $csmnode, $hostname);
    my $csmcfgdir = "/csminstall/csm/config";
    my $nodemap_file = "/csminstall/csm/config/nodemap";
    my $node;
    my $dshresult="";
    #
    #  Check and/or create the /csminstall/csm/config directory
    #
    if (! -d $csmcfgdir) {
        $cmd = "$::MKDIR -m 755 -p $csmcfgdir";
        $rc = system("$cmd");
        if ($rc >> 8) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_CreateDir', $csmcfgdir);
            return($::NOK);
        }
    }
    # Remove any old occurences of the nodemap file
    if (-f $nodemap_file) {
        $cmd = "$::RM $nodemap_file";
        $rc = system("$cmd") >> 8;
        if ($rc != 0) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_RM', "$nodemap_file");
            # try to continue
        }
    }
    # open the map file
    unless (open(NODEMAP,">$nodemap_file")) {
        # print error message
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', "$nodemap_file");
        return $::NOK;
    }
    my @AIX;
    my @Linux;
    my %found = ();
    # for each node in the node list
    foreach $node (keys %DestNodeHash) {
	# set the correct command based on the InstallOSName
        if ( defined($DestNodeHash{$node}{'InstallOSName'}) ) {
            if ($DestNodeHash{$node}{'InstallOSName'} eq "AIX") {
                #$dshcmd = "$::DSH -n$node hostname";
                push @AIX, $node;
            } elsif ($DestNodeHash{$node}{'InstallOSName'} eq "Linux") {
                #$dshcmd = "$::DSH -n$node hostname -f";
                push @Linux, $node;
            }
	} 
    }
    if(@Linux){
	my $node_file = InstallUtils->make_node_list_file(\@Linux); #put all dest nodes in tmp file
	$ENV{'DSH_LIST'} = $node_file;
	my $dshcmd = "$::DSH hostname -f";
	my @dshresult = `$dshcmd`;
	$rc = $? >> 8;
	InstallUtils->close_delete_file($::NODE_LIST_FILE, $node_file); #delete node file
	if ($rc != 0) {
	    # print error message
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', $dshcmd, $rc);
	    #$main::dsh_failed{$node}=1; #take these out of target nodes
	    #next;
        }
	foreach my $line (@dshresult){
	    my ($csmnode, $hostname) = split(" ",$line);
	    $hostname     =~ s/://g;
	    $csmnode      =~ s/://g;
	    if ( defined($csmnode) && defined($hostname) ) {
		# make sure the format is consistent with the name that
		#     was used to create the configinfo file
		$csm_nodename = NodeUtils->tryHost($csmnode);
		$found{$csm_nodename}=1;	
		# put the result in the nodemap file
		unless(print NODEMAP "$csm_nodename $hostname\n")  {
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_WRITEtoFILE', "$nodemap_file");
		}
	    }
	}
    }
    if(@AIX){
	my $node_file = InstallUtils->make_node_list_file(\@AIX); #put all dest nodes in tmp file
	$ENV{'DSH_LIST'} = $node_file;
	my $dshcmd = "$::DSH hostname";
	my @dshresult = `$dshcmd`;
	$rc = $? >> 8;
	InstallUtils->close_delete_file($::NODE_LIST_FILE, $node_file); #delete node file
	if ($rc != 0) {
	    # print error message
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', $dshcmd, $rc);
	    #$main::dsh_failed{$node}=1; #take these out of target nodes
	    #next;
	}
	foreach my $line (@dshresult){
	    my ($csmnode, $hostname) = split(" ",$line);
	    $hostname     =~ s/://g;
	    $csmnode      =~ s/://g;
	    if ( defined($csmnode) && defined($hostname) ) {
		# make sure the format is consistent with the name that
		#     was used to create the configinfo file
		$csm_nodename = NodeUtils->tryHost($csmnode);
		$found{$csm_nodename}=1;	
		# put the result in the nodemap file
		unless(print NODEMAP "$csm_nodename $hostname\n")  {
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_WRITEtoFILE', "$nodemap_file");
		}
	    }
	}
    }
    foreach my $n (keys %DestNodeHash){
	if(!$found{$n}){
	    $main::dsh_failed{$n}=1;   
	}
    }
    close (NODEMAP);
    if (!@AIX && !@Linux){
        return $::NOK;
    }
    return $::OK;
}

#-------------------------------------------------------------------------#
#                                                                         #
#   start_nfs                                                             #
#                                                                         #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub start_nfs{
    my ($cmd, $output);
    my $rc=0;
    my $exports = "/etc/exports";
    #my $PLTFRM = `uname`; chomp $PLTFRM;      #using $::PLTFRM instead

    # Ensure nfs is running.
    if (! -s $exports){ 
        # nfs will only start if /etc/exports has a size > 0
        open (EXPORTS, ">$exports") || NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_READ_FILE', $exports); 
        print EXPORTS "\n";
        close (EXPORTS);
    }

    # Start nfs if needed.  
    if ($::PLTFRM eq "AIX") {
        my $cmd = "$::MKNFS -B  >/dev/null 2>&1";
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCMD', $cmd);
        my $rc  = system ($cmd) >> 8;
        if ($rc) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        }

    } else {
        # We need modify start_nfs in rc.config on SuSE
        my $distro=NodeUtils->distribution();
        if( $distro =~ /SuSE/){
                my $RCCONFIG="/etc/rc.config";
                my $s=NodeUtils->readFile($RCCONFIG);
                $s=~s/^\s*NFS_SERVER\s*=\s*"\s*[Nn][Oo]\s*"\s*$/NFS_SERVER="yes"/m;
                if( open(OUT, ">$RCCONFIG")){
                        print OUT ($s);
                        close(OUT);
                }
                    else{
                        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgShow_Output','Can not open $RCCONFIG for write');
                        }
        }

        # The output of the 'service nfs probe' tells 
        #    what needs to be done to start nfs.  It returns either 
        #    'start', 'restart', 'reload' or nothing.

        # SuSE Change
        #$output = `$::SERVICE nfs probe 2>&1`;
        $output=NodeUtils->nfsprobe();

        if ($output){
            # SuSE Change
            #$cmd = "$::SERVICE nfs $output";
            $cmd=NodeUtils->service("nfs",$output);

            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgRUN_CMD', $cmd);
            $rc  = system ($cmd) >> 8;
            if ($rc) { 	
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCommandFailed', $cmd, $rc);
            }
            else{
                #wait for nfsserver to start on SLES7
                sleep(1); 
            }
        }
    }
    return ($rc);
}

#-------------------------------------------------------------------------#
#                                                                         #
#   export_csminstall                                                     #
#                                                                         #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub export_csminstall{
    my $cmd;
    my $rc=0;
    #my $PLTFRM = `uname`; chomp $PLTFRM;    #using $::PLTFRM instead
    
    if ($::PLTFRM eq "AIX") {
        $cmd = "$::EXPORTFS -i -o ro /csminstall";
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCMD', $cmd);
        $rc  = system ($cmd) >> 8;
        if ($rc) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        }

    } else {

        $cmd = "$::EXPORTFS -o ro,no_root_squash \*:/csminstall";
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgRUN_CMD', $cmd);
        $rc  = system ($cmd) >> 8;
        if ($rc) { 
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCommandFailed', $cmd, $rc);
        }
    }

    return ($rc);
}

#-------------------------------------------------------------------------#
#                                                                         #
#   unexport_csminstall                                                   #
#                                                                         #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub unexport_csminstall{
    my $cmd;
    my $rc=0;
    #my $PLTFRM = `uname`; chomp $PLTFRM;      #using $::PLTFRM instead

    if ($::PLTFRM eq "AIX") {
        $cmd = "$::EXPORTFS -i -u /csminstall";
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCMD', $cmd);
        $rc  = system ($cmd) >> 8;
        if ($rc) { 
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        }
    } else {

        $cmd = "$::EXPORTFS -u \*:/csminstall";
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgRUN_CMD', $cmd);
        $rc  = system ($cmd) >> 8;
        if ($rc) { 
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCommandFailed', $cmd, $rc);
        }
    }
    return ($rc);
}

###########################################################	
# Get the hostname of the management server.
#
# Since the nodes might need to contact the management server through a
# different address than its primary hostname, we cannot just use the output
# of the hostname command here.  Use the getSourceIP2Target command instead.
#
# If we cannot determine the route to the node via getSourceIP2Target
# just return nothing.  This indicates that there is no way to get to the node.
###########################################################

sub get_management_server
{
    my ($class, $node_hostname) = @_;
    my $ipaddr   = "";
    my $hostname = "";

    # Move this to InstallDefs.pm.perl
    $::GETSOURCEIP2TARGET = "/opt/csm/csmbin/getSourceIP2Target";

    my $cmd = "$::GETSOURCEIP2TARGET $node_hostname";
    my @output = NodeUtils->runcmd($cmd,0);
    my $rc = $::RUNCMD_RC;
    if (! $rc) # If no errors
    {
	$ipaddr = $output[0];

	if($ipaddr){
	    $hostname = NodeUtils->tryHost($ipaddr);
	}
    }

    if (wantarray) { return ($hostname, $ipaddr); }
    else { return $hostname; }
}

#################################################
#The following two subroutines are used for locking the tool
#by obtaining a lock at the beginning of execution and holding
#it until the tool completes. Therefore it is guaranteed that two or more
#instances of updatenode will NOT be able to be run.
#
#If process A invokes get_lock and process B invokes it after 
#process A has, process B will fail. 
#
#In the event that the program is aborted, the lock is automatically
#released.
#
#INPUT:
#	get_lock takes the filename you want to use as an argument
#OUTPUT:
#	returns a 2 if it couldn't open the file to lock it
#	returns a 3 if there was an error locking the file
###########################################################

sub get_lock 
{
    my ($class, $file) =@_;
    my $return_code = 0;
    my $LOCK_EX = 2;

    if(!open(SEM,">$file")){ 
        $return_code = 2;
    }

    if(!flock(SEM,$LOCK_EX)){ 
        $return_code =3;
    }
    return $return_code;
}

#-------------------------------------------------------------------------#
#                                                                         #
#   release lock							  #
#                                                                         #
#       Releases the lock of the file of get lock sub routine             #
#-------------------------------------------------------------------------#
sub release_lock
{
    close(SEM);
}

#-------------------------------------------------------------------------#
#                                                                         #
#   backup_logfile                                                        #
#                                                                         #
#        Backup the current logfile. Move logfile to logfile.1.           #
#        Shift all other logfiles (logfile.[1-3]) up one number.          #
#        The original logfile.4 is removed.                               #
#                                                                         #
#-------------------------------------------------------------------------#
sub backup_logfile
{
    my ($logfile) = @_;

    my ($logfile1) = $logfile . ".1";
    my ($logfile2) = $logfile . ".2";
    my ($logfile3) = $logfile . ".3";
    my ($logfile4) = $logfile . ".4";

    if (-f $logfile) {

        rename ($logfile3, $logfile4) if (-f $logfile3);
        rename ($logfile2, $logfile3) if (-f $logfile2);
        rename ($logfile1, $logfile2) if (-f $logfile1);
        rename ($logfile,  $logfile1) if (-f $logfile);
    }
    return $::OK;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    start_logging                                                        #
#                                                                         #
#           Start logging messages to a logfile. Return the log file      #
#           handle so it can be used to close the file when done logging. #
#                                                                         #
#-------------------------------------------------------------------------#
sub start_logging
{
    my ($class, $logfile) = @_;
    my ($cmd, $rc);

    &backup_logfile($logfile);

    # create the log directory if it's not already there
    if (! -d $::CSMLOG) {
        $cmd = "$::MKDIR -m 644 -p $::CSMLOG";
        $rc = system("$cmd");
        if ($rc >> 8) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_CreateDir', $::CSMLOG);
            return($::NOK);
        }
    }

    # open the log file
    unless (open(LOGFILE, ">$logfile")) {
        # Cannot open file
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', $logfile);
        return $::NOK;
    }

    $::LOG_FILE_HANDLE = \*LOGFILE;

    # Print the date to the top of the logfile
    my $sdate = `/bin/date`;
    chomp $sdate;
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"I", 'IMsgWRITING_LOGFILE', $logfile);
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"LI", 'IMsgSTART_LOGGING', $sdate);

    return ($::LOG_FILE_HANDLE);
}

#-------------------------------------------------------------------------#
#                                                                         #
#    append_logging                                                       #
#                                                                         #
#         Append logging messages to a logfile. Return the log file       #
#           handle so it can be used to close the file when done logging. #
#                                                                         #
#-------------------------------------------------------------------------#
sub append_logging
{
    my ($class, $logfile) = @_;
    my ($cmd, $rc);

    # create the log directory if it's not already there
    if (! -d $::CSMLOG) {
        $cmd = "$::MKDIR -m 644 -p $::CSMLOG";
        $rc = system("$cmd");
        if ($rc >> 8) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_CreateDir', $::CSMLOG);
            return($::NOK);
        }
    }

    #  
    #  get log file ready
    #
    if (! -e $logfile) {  #  create the log file if not already there
        # open the log file
        unless (open(LOGFILE, ">$logfile")) {
            # Cannot open file
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', $logfile);
            return $::NOK;
        }
    } else {   # it's there so just append
        unless (open(LOGFILE, ">>$logfile")) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', $logfile);
            return $::NOK;
        }
    }

    $::LOG_FILE_HANDLE = \*LOGFILE;

    # Print the date to the top of the logfile
    my $sdate = `/bin/date`;
    chomp $sdate;
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"I", 'IMsgWRITING_LOGFILE', $logfile);
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"LI", 'IMsgSTART_LOGGING', $sdate);

    return ($::LOG_FILE_HANDLE);
}

#-------------------------------------------------------------------------#
#                                                                         #
#    stop_logging                                                         #
#                                                                         #
#                  Turn off message logging.  (Expects to have a file     #
#                   handle passed in. )                                   #
#                                                                         #
#-------------------------------------------------------------------------#
sub stop_logging
{
	if (defined($::LOG_FILE_HANDLE)) {
    		# Print the date at the bottom of the logfile
    		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"LI", 'IMsgSTOP_LOGGING', `/bin/date`);

    		close($::LOG_FILE_HANDLE);
    		undef $::LOG_FILE_HANDLE;
	}
    return $::OK;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    get_OSName                                                           #
#                                                                         #
#                  Returns the Os name i.e. Linux , AIX else undefined    #
#                                                                         #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_OSName
{
    if(NodeUtils->isLinux()==1){
	    return "Linux";
    }
    elsif(NodeUtils->isAIX()==1){
	    return "AIX"
    }
    return undef;
}

#------------------------------------------------------------------------#
#                                                                        #
#    DistributionVersion                                                       #
#                                                                        #
#        Returns the Operating System version that is installed on the   #
#        management server.                                              #
#        For AIX, the version is in the form V.R.M.  For Linux, it is    #
#        in the form reported in the /etc/redhat-release file (V.R).     #
#------------------------------------------------------------------------#
sub get_DistributionVersion
{
    my ($cmd, $output, $rpm, $version, $release, $v1, $v2, $v3, $rest);

    if ($::PLTFRM eq "Linux")
    {
        
        $version = (split ' ', (NodeUtils->distribution()))[1];
        
        #if (-f '/etc/redhat-release')
        #{
        #    my $s = NodeUtils->readFile('/etc/redhat-release');
        #    ($version) = $s =~ /\s(\d+\.\d+)\s/;
        #}
    }
    elsif ($::PLTFRM eq "AIX")
    {
        $cmd = "$::OSLEVEL";
        $output = NodeUtils->runcmd($cmd);
        chomp ($output);

        ($v1, $v2, $v3, $rest) = split(/\./, $output);
        $version = join (".", $v1, $v2, $v3);
    }
    else
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
    }

    return (length($version) ? $version : undef);
}

#------------------------------------------------------------------#
#                                                                  #
#    get_CSMVersion                                                #
#                                                                  #
#       Get the version of csm component that is installed on the  #
#       management server.                                         #
#       The version should be in the form "V.R.M" (i.e 1.2.0).     #
#                                                                  #
#                                                                  #
#------------------------------------------------------------------#
sub get_CSMVersion
{
    my ($class, $component) = @_;
    my ($cmd, $output, $rpm, $version, $release, $v1, $v2, $v3, $rest);

    if ($::PLTFRM eq "Linux")
    {
        $cmd = "$::RPMCMD -q $component";
        $output = NodeUtils->runcmd($cmd);
        chomp ($output);
        ($rpm, $version, $release) = split (/-/, $output);

        ($v1, $v2, $v3, $rest) = split(/\./, $version);
        $version = join (".", $v1, $v2, $v3);
    }
    elsif ($::PLTFRM eq "AIX")
    {
        $cmd = "$::LSLPP -cLq $component";
        $output = NodeUtils->runcmd($cmd);
        chomp ($output);

        # The third field in the lslpp output is the VRMF
        ($version) = (split(/:/, $output))[2];

        ($v1, $v2, $v3, $rest) = split(/\./, $version);
        $version = join (".", $v1, $v2, $v3);
    }
    else
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
    }

    return (length($version) ? $version : undef);
}

#-------------------------------------------------------------------------#
#                                                                         #
#    get_PkgArchitecture                                                  #
#                                                                         #
#        Returns the Os Architure of the OS i.e. i386 for linux           #
#        This only for Linux                                              #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_PkgArchitecture
{
  my ($cmd, $output, $osarch);
 if ($::PLTFRM eq "Linux")
 {
    $cmd = "$::UNAME -m";
    $output = NodeUtils->runcmd($cmd);
    chomp ($output);

    $osarch = $output;

   return (length($osarch) ? $osarch : undef);
 }
 elsif ($::PLTFRM eq "AIX")
 {
     $osarch="";
    return (length($osarch) ? $osarch : undef);
 }
 else
 {
   NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
 }
}

#-------------------------------------------------------------------------#
#                                                                         #
#    get_pkgVersion                                                       #
#                                                                         #
#       Returns the installp image version from Installp command which,   #
#        looks into the .toc file.                                        #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_pkgVersion
{
   my ($class, $pkg) = @_;
   my ($cmd, $output, $version);

   if ($::PLTFRM eq "Linux")
   {
        $version = "";
      return (length($version) ? $version : undef);
   }
   elsif ($::PLTFRM eq "AIX")
   {
      $cmd=qq($::INSTALLPCMD -L -d . | grep $pkg 2>&1);
      $output = NodeUtils->runcmd($cmd);
      chomp ($output);
      (undef,undef,$version,undef)=split(/:/,$output);
      return (length($version) ? $version : undef);
   }
   else
   {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
   }

}

#-------------------------------------------------------------------------#
#                                                                         #
#    get_DistributionName                                                 #
#                                                                         #
#       Get the distribution name for linux its RedHat and return         # 
#       nothing for AIX.						  #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_DistributionName
{
    my ($cmd, $output, $rpm, $distro);

    # IF LINUX
    if ($::PLTFRM eq "Linux")
    {
        $distro = (split ' ', (NodeUtils->distribution()))[0];
        
        #if (-f '/etc/redhat-release')
        #{
            # RedHat is the only Linux distro we support for now.
        #    $distro = "RedHat";
        #}
    }
    elsif ($::PLTFRM eq "AIX")
    {
        $distro = "";
    }
    else
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
    }

    return (length($distro) ? $distro : undef);
}

#-------------------------------------------------------------------------#
#                                                                         #
#    get_uniqu_arry_elemnts                                               #
#                                                                         #
#       Returns the unique elements in an array                           #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_uniqu_arry_elemnts
{
   my ($class, @my_arry) =@_;
   my %my_hsh=();
   my @uniq=();
   my $itm;
   foreach $itm (@my_arry)
   {
     $my_hsh{$itm}++;
   }
   @uniq= keys %my_hsh;
   return @uniq;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    findRpmInPath                                                        #
#                                                                         #
#       Find the rpm package in the given path                            #
#                                                                         #
#-------------------------------------------------------------------------#
sub findRpmInPath
{
   my ($class, $rpmfile, @path) = @_;
   foreach my $p (@path)
   {
     my $filename = "$p/$rpmfile";
     # use the ls command here instead of the -e operator so we can handle *s in the names
     #print "looking for $filename: ", `/bin/ls $filename`, "\n";
     if (`/bin/ls $filename 2> /dev/null`)
     {
       NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgFOUND_RPM', $rpmfile, $p);
       return $p;
     }
   } 
   return undef;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    findRpm                                                              #
#                                                                         #
#       Try to find and return the rpm file that is the required version  #
#                                                                         #
#-------------------------------------------------------------------------#
sub findRpm
{
        my ($class, $pkg, $operator, $version, @my_path) = @_;

        my $os_rpm_path=$my_path[0];
        my $pkg_rpm_path=$my_path[1];
     if ($::PREREQS_ATTR{'OSName'} eq "Linux")
     {
        # Get the current distribution if it was not already obtained.
        my $distro = $::DISTRO;
        if (! $distro)
        {
            $distro = &get_DistributionName." ".&get_DistributionVersion;
            if (!grep(/^$distro$/, @::VALID_DISTROS))
            {
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E5', 'EMsgUNSUPPORTED_DISTRO', $distro, join(', ',@::VALID_DISTROS));
            }
            else
            {
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDETECTED_DISTRO', $distro);
            }
        }
      }
       # note: The -vr options on the ls command gives us the entries in
        #       reverse version order to get the latest version first.
        my @rpms;
        my @rpms1;
        my @rpms2;

        # Look for the rpm in packages
        if (NodeUtils->isLinux)
        {
           @rpms1 = NodeUtils->runcmd("$::LS -vr $pkg_rpm_path/$pkg-*", -1);
        }
        else
        {
           @rpms1 = NodeUtils->runcmd("$::LS  $pkg_rpm_path/$pkg-*", -1);
        }
        my $rc1 = $::RUNCMD_RC;
        if (! $rc1) { @rpms = (@rpms, @rpms1); } # Only add if no errors

        # Look for the rpm in  RPMS
        #my $TFTPBOOT_DISTRORPM = "$::TFTPBOOT/$::DISTRO_SHORTNAME{$distro}/$::DISTRO_RPMDIR{$distro}";
        if (NodeUtils->isLinux)
        {
           @rpms2 = NodeUtils->runcmd("$::LS -vr $os_rpm_path/$pkg-*", -1);
        }
        else
        {
           @rpms2 = NodeUtils->runcmd("$::LS $os_rpm_path/$pkg-*", -1);
        }
        my $rc2 = $::RUNCMD_RC;
        if (! $rc2) { @rpms = (@rpms, @rpms2); } # Only add if no errors

        if (($rc1 && $rc2) || !scalar(@rpms)) { return undef; }

        foreach my $r (@rpms)
          {
                #my ($ipkg, $iversion, $irelease) = split('-', $r);
                my ($fn) = $r =~ m|^.*?([^/]+)$|;      # get rid of the directory, if any
                #print "fn=$fn\n";
                my ($ipkg, $iversion, $irelease) = $fn =~ /^(.+)-(\d.*)-(.*)$/;
                #print "ipkg=$ipkg, iversion=$iversion, irelease=$irelease\n";
                # Note: irelease has .i386.rpm on the end of it
                if ($ipkg ne $pkg || !length($iversion)) { next; }
                if (!length($operator) || !length($version) || InstallUtils->testVersion($iversion, $operator, $version))
                  {
                        $operator=" " if (!$operator);
                        $version=" " if (!$version);

                        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgFOUND', $r, $operator, $version);
                        return $r;
                  }
          }
        return undef;   # didn't find a suitable rpm

}

#-------------------------------------------------------------------------#
#                                                                         #
#   testVersion                                                           #
#                                                                         #
#       Compare version1 and version2 according to the operator and       #
#       return true or false. These expressions come from the Require     #
#       statement of our rpms.						  #
#                                                                         #
#-------------------------------------------------------------------------#
sub testVersion
  {
	my ($class, $version1, $operator, $version2, $release1, $release2) = @_;

	# Note: perl seems to be able to handle really big integers, but if we run into a limit, we
	#       can use the Math::BigInt pkg

	my @a1 = split(/\./, $version1);
	my @a2 = split(/\./, $version2);
	my $len = (scalar(@a1)>scalar(@a2) ? scalar(@a1) : scalar(@a2));
	$#a1 = $len -1;      # make the arrays the same length before appending release
	$#a2 = $len -1;
	push @a1, split(/\./, $release1);
	push @a2, split(/\./, $release2);
	$len = (scalar(@a1)>scalar(@a2) ? scalar(@a1) : scalar(@a2));
	my $num1 = '';
	my $num2 = '';

	for (my $i=0; $i<$len; $i++)
	  {
		my ($d1) = $a1[$i] =~ /^(\d*)/;     # remove any non-numbers on the end
		my ($d2) = $a2[$i] =~ /^(\d*)/;
		
		my $diff = length($d1) - length($d2);
		if ($diff > 0)     # pad d2
		  {
			$num1 .= $d1;
			$num2 .= ('0' x $diff) . $d2;
		  }
		elsif ($diff < 0)     # pad d1
		  {
			$num1 .= ('0' x abs($diff)) . $d1;
			$num2 .= $d2;
		  }
		else        # they are the same length
		  {
			$num1 .= $d1;
			$num2 .= $d2;
		  }
	  }

	# Remove the leading 0s or perl will interpret the numbers as octal
	$num1 =~ s/^0+//;
	$num2 =~ s/^0+//;

	if (length($release1)) { $release1 = "-$release1"; }   # this is just for error msgs
	if (length($release2)) { $release2 = "-$release2"; }   # this is just for error msgs
	
	if ($operator eq '=') { $operator = '=='; }
	my $bool = eval "$num1 $operator $num2";
	if (length($@))
	  {
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','W', 'EMsgVERSION_COMPARE_PROBLEM', "$version1$release1 $operator $version2$release2", $@);
		return undef;
	  }
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCOMPARING_VERSION', "$version1$release1 $operator $version2$release2", ($bool?'true':'false'));
	return $bool;
  }

#-------------------------------------------------------------------------#
#                                                                         #
#    installRequiredRpms                                                  #
#                                                                         #
#     Install any rpms that this rpm is dependent on if they              #
#     are not already on the system at the required level.                #
#     (The rpm passed in is the whole rpm file name.)                     #
#                                                                         #
#-------------------------------------------------------------------------#
sub installRequiredRpms
{
        my ($class, $rpm, @my_path) = @_;
        my @reqs = InstallUtils->getRequiredRpms($rpm, @my_path);
        if (scalar(@reqs))        # need to install at least 1 rpm
          {
                my $reqlist = join(' ', @reqs);
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgINSTALLING_RPM_REQS', $rpm, $reqlist);
                my ($cmd) = "$::RPMCMD -U $reqlist";
                $cmd = "$cmd --force" if $::FORCE_INSTALL;
                NodeUtils->runcmd("$cmd");
          }

}

#-------------------------------------------------------------------------#
#                                                                         #
#    getRequiredRpms                                                      #
#                                                                         #
#       Return the names of any rpms that this rpm is dependent on if     #
#       they are not already on the system at the required level.         #
#       The rpm passed in as the 1st arg is the whole rpm file name.      # 
#       The rest of the args are the list of reqs we have already found   #
#       in previous, recursive invokations of the function.               #
#                                                                         #
#-------------------------------------------------------------------------#
sub getRequiredRpms
{
        my ($class, $rpm, @my_path) = @_;
        my @reqrpms;
        my @reqs = InstallUtils->getReqsNotSatisfied($rpm);
        #print "num=" . scalar(@reqs) . ' ,reqs=' . join(':', @reqs) . ".\n";
        foreach my $r (@reqs)
          {
                my ($pkg, $operator, $version) = split(' ', $r);

                my $newrpm = InstallUtils->findRpm($pkg, $operator, $version, @my_path);
                if (defined($newrpm))        # found an appropriate one to install
                  {
                        # make sure it is not already on the list
                        if (grep(/^$newrpm$/, @_)) { next; }   # already on the list

                        push @reqrpms, $newrpm;

                        push @reqrpms, InstallUtils->getRequiredRpms($newrpm, @reqrpms, @my_path);
                  }
                else { NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E11', 'EMsgCANT_FIND_RPM', $pkg, $operator, $version, $rpm); }
          }
        return @reqrpms;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    getReqsNotSatisfied                                                  #
#                                                                         #
#       Return the rpms (and versions) that the given rpm requires        #
#       that are not already on the system                                #
#                                                                         #
#-------------------------------------------------------------------------#
sub getReqsNotSatisfied
{
        my ($class, $rpm) = @_;
        my @requires;
        my @output = NodeUtils->runcmd("$::RPMCMD -i --test $rpm", -1);
        if ($::RUNCMD_RC == 0) { return @requires; }    # all dependencies were satisfied
        if ($::RUNCMD_RC > 1) { return @requires; }
        my $firstline = shift @output;
        if ($firstline=~/is already installed/) { return @requires; }
        if (!($firstline=~/failed dependencies/)) { return @requires; }

        foreach my $o (@output)
          {
                #print "o=$o\n";
                my ($pkg, $operator, $version) = InstallUtils->parseRequire($o);
                #print "pkg=$pkg, operator=$operator, version=$version\n";
                if (!defined($pkg)) { next; }   # was not an rpm require

                # Hack for glibc version because I can not find a way to have the require
                # statement in the rpm be different for different distribution levels.
                if ($pkg eq 'glibc' && NodeUtils->distribution()=~/^RedHat 7/i)
                  { $o = "$pkg $operator 2.2"; }

                my $req = "$pkg $operator $version";
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgREQUIRE', $rpm, $req);
                push(@requires, $req);
          }
        return @requires;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    parseRequire                                                         #
#                                                                         #
#       Parse a single require line, distinguishing between rpms and      #
#       individual libraries of executables.                              #
#                                                                         #
#-------------------------------------------------------------------------#
sub parseRequire
{
        my ($class, $line) = @_;
        $line =~ s/ is needed by .*$//;
        my ($pkg, $operator, $version, $therest) = split(' ', $line);
        if ($pkg=~/^rpmlib/ || $pkg=~m|/| || $pkg=~/\.so/) { return undef; }
        else { return ($pkg, $operator, $version); }
}


#-------------------------------------------------------------------------#
#									  #
#      inc_ip								  #
#           Increment an IP address by 1, carrying to the previous octet  #
#           as required. This code originally appeared in the LUI 	  #
#           product (luiapi.pm)						  #
#                                                                         #
#-------------------------------------------------------------------------#
sub inc_ip
{
    my $ip=$_[1];
    my ($a,$b,$c,$d);
    ($a,$b,$c,$d)=split(/\./,$ip);
    $d++;
    if ($d > 255) {
        $d = $d - 256;
        $c++;
        if ($c > 255) {
            $c = $c - 256;
            $b++;
            if ($b > 255) {
                $b = $b - 256;
                $a++;
                if ($a > 255) {
                    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgIPADDR_ERROR', "$a.$b.$c.$d");
                    #printf "Error - ip address out of valid range.\n";
                    return 1;
                }
            }
        }
    }
    return $a."\.".$b."\.".$c."\.".$d;

}



#-------------------------------------------------------------------------#
#									  #
#      genHex								  #
#      generates a hex representation of an ip address:                   #
#      for example, 9.114.133.193 --> 097285C1                            #
#      Takes in a list of hosts                                           #
#      and returns a hex where the key is the hostname and                #
#                             the value is the IP in hex form.            #
#-------------------------------------------------------------------------#

sub genHex{
    shift;
    my $host;
    my $uno;
    my $dos;
    my $tres;
    my $cuatro;
    my $hex;
    my %hex;
    foreach $host (@_){
        my $ip =  NodeUtils->getHost($host);            # Get ip address
        ($uno, $dos, $tres, $cuatro) = split(/\./, $ip);# Split by the .
        $hex = sprintf("%02X", $uno);                   # put formatted into hex
        $hex .= sprintf("%02X", $dos);
        $hex .= sprintf("%02X", $tres);
        $hex .= sprintf("%02X", $cuatro); 
        $hex{$host} = $hex;
    }
    return %hex;
}

#-------------------------------------------------------------------------#
#                                                                         #
#      setup_atftp                                                        #
#                                                                         #
#      Modify /etc/inetd.conf for atftp                                   # 
#                                                                         # 
#                                                                         # 
#-------------------------------------------------------------------------#
sub setup_atftp_inetd
{
    my $inetdconf         = "/etc/inetd.conf";
    my $inetdbackup   = "/tmp/tftp_inetd.bak";
    my $errors         = 0;
    my ($cmd, $output, $rc,$s);


    if (not -f $inetdconf ){
            #NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgINETD_MISSING', "$inetdconf");
            $errors++;
    }
    else{

    $s=NodeUtils->readFile($inetdconf);

    #comment tftp entry
    $s=~s/^\s*(tftp\s+[\s\S]*$)/#$1/m;

    $s=$s."\ntftp   dgram   udp     wait    nobody  /usr/sbin/tcpd  /usr/sbin/in.tftpd --tftpd-timeout 300  --retry-timeout 5 --maxthread 100 --verbose=5 --logfile /var/log/atftpd.log /tftpboot\n";

    # backup inetd.conf
    $cmd = "$::COPY -f $inetdconf $inetdbackup ";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    if ($rc)
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgShow_Output',$output);
        $errors++;
    }

    if( open(OUT, ">$inetdconf")){
        print OUT ($s);
        close(OUT);
    }
    else{
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgShow_Output','Can not open $inetdconf for write');
        $errors++;
    }

    # Restart xinetd to pick up the changes
        $cmd=NodeUtils->service("inetd","start");
        NodeUtils->runcmd($cmd,0);
        if($rc){
            $errors++;
        }
        $cmd=NodeUtils->service("inetd","reload");
        NodeUtils->runcmd($cmd,0);
        if($rc){
            $errors++;
        }

    }

    return $errors;

}

#-------------------------------------------------------------------------#
#									  #
#      setup_atftp  						          #
#									  #
#	   Create the /etc/xinetd.d/tftp file to work with atftp.         #
#									  #
#									  #
#-------------------------------------------------------------------------#
sub setup_atftp
{
    my $xinetd         = "/etc/xinetd.d";
    my $xinetdfile     = $xinetd . "/tftp";
    my $xinetdbackup   = "/tmp/tftp.bak";
    my $atftp_conf     = "/opt/csm/install/atftp_xinetd.conf";
    my $errors         = 0;
    my $DIFF           = "/usr/bin/diff";
    my $inetdconf      = "/etc/inetd.conf";
    my ($cmd, $output, $rc);

    # SuSE Change
    # Check inetd in SuSE and donnot check xinetd
    my $distro=NodeUtils->distribution();
    if( $distro =~ /SuSE/){
        my $inetderr=setup_atftp_inetd();
        return $inetderr;
    }
    else {

      unless (-d $xinetd ){
          NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgXINETD_MISSING', "$xinetd");
          $errors++;
      }

      # Only copy the file if it is different
      $cmd = "$DIFF $atftp_conf $xinetdfile 2>&1";
      $output = NodeUtils->runcmd($cmd,-1);
      $rc = $::RUNCMD_RC;
      if ($rc)
      {
          # Back up any existing file
          if (-f $xinetdfile)
          {
	          NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"I", 'IMsgCHANGING_TFTP_XINETD_FILE');
    	      $cmd = "$::COPY -f $xinetdfile $xinetdbackup";
    	      $output = NodeUtils->runcmd($cmd,0);
    	      $rc = $::RUNCMD_RC;
      	      if ($rc)
	      {
		          #print $output;
                  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgShow_Output',$output);
		          $errors++;
	      }
	  }

    	  # Now, copy our version of the file
    	  $cmd = "$::COPY -f $atftp_conf $xinetdfile";
	      $output = NodeUtils->runcmd($cmd,0);
      	  $rc = $::RUNCMD_RC;
	  if ($rc)
	  {
	      #print $output;
              NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgShow_Output',$output);
	      $errors++;
	   }

	   # Restart xinetd to pick up the changes
           # SuSE Change
	   #`$::SERVICE xinetd start`;
	   #`$::SERVICE xinetd reload`;
           $cmd=NodeUtils->service("xinetd","start");
           NodeUtils->runcmd($cmd,0);
           if($rc){
                $errors++;
           }
          $cmd=NodeUtils->service("xinetd","reload");
          NodeUtils->runcmd($cmd,0);
          if($rc){
                $errors++;
          }
      }

      return $errors;
    }
}

#-------------------------------------------------------------------------#
#                                                                         #   
#      verify_tftp                                                        #
#                                                                         #
#      check that tftp server is running in /etc/inetd.conf.  Also check  #
#          that the client program is found in $::ATFTP.                  #
#                                                                         #
#                                                                         #
#-------------------------------------------------------------------------#
sub verify_tftp_inetd
{
    my $errors = 0;
    my $cmd;
    #NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCheckingAtftp');
    if(-e $::ATFTP){
        #NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgfoundAtftp', $::ATFTP);
        my $version = `$::ATFTP -V 2>/dev/null`;              # see what version

        chomp(my @output = `$::LSOF | $::GREP tftp | $::GREP inetd`);
        if(@output){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpRunning');
        }
        else{
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpNotUp');
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpTryStart');
            $cmd=NodeUtils->service("inetd","start");
            `$cmd`;
            $cmd=NodeUtils->service("inetd","reload");
            `$cmd`;
            chomp(my @output = `$::LSOF | $::GREP tftp | $::GREP inetd`);
            if(@output){
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpNowRunning');
            }
            else{
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpCantUp');
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpInetdExistquest');

                my $inetdconf         = "/etc/inetd.conf";
                my ($output,$s);

                $s=NodeUtils->readFile($inetdconf);

                #comment tftp entry
                if( $s=~/^\s*(tftp\s+[\s\S]*$)/){;
                    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpInetdExist');
                }
                else{ NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpUrProblem'); }
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpProblemHand');
                $errors++;
            }
        }

    }
    else{
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I', 'IMsgAtftpNotFound',$::ATFTP);
        $errors++;
    }

    return $errors;
}


#-------------------------------------------------------------------------#
#									  #
#      verify_tftp  						          #
#									  #
#	   check that tftp server is running in /etc/xinetd.  Also check  #
#          that the client program is found in $::ATFTP.                  #
#									  #
#									  #
#-------------------------------------------------------------------------#

sub verify_tftp 
{
    my $errors = 0;
    my $cmd;

    # SuSE Change
        # Check the inetd in SuSE first to prevent unnecessary error messages.
        # And donnot check xinetd in SuSE
    my $distro=NodeUtils->distribution();
    if( $distro =~ /SuSE/){
        my $inetderr= verify_tftp_inetd();
                return $inetderr;
    }
    else {

	 NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCheckingAtftp');
	 if(-e $::ATFTP){
	     NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgfoundAtftp', $::ATFTP);
             my $version = `$::ATFTP -V`;              # see what version

             chomp(my @output = `$::LSOF | $::GREP tftp | $::GREP xinetd`);
              if(@output){
            	  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpRunning');
       	      }
	      else{
                  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpNotUp');
                  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpTryStart');
                  # SuSE Change
                  #`$::SERVICE xinetd start`;
                  #`$::SERVICE xinetd reload`;
                  $cmd=NodeUtils->service("xinetd","start");
                  `$cmd`;
                  $cmd=NodeUtils->service("xinetd","reload");
                  `$cmd`;
                   chomp(my @output = `$::LSOF | $::GREP tftp | $::GREP xinetd`);
                   if(@output){
                      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpNowRunning');
                   }
                   else{
                       NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpCantUp');
                       NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpXinetdExistquest');
                       if(-f "/etc/xinetd.d/tftp"){
                          NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpXinetdExist');
                       }
                       else{ NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpUrProblem'); }
                       NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgAtftpProblemHand');
                       $errors++;
                   }
	      }

	 }
         else{
             NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I', 'IMsgAtftpNotFound',$::ATFTP);
             $errors++;
         }
    
         return $errors;
    }
}

#-------------------------------------------------------------------------#
#									  #
#      verify_dhcp  						          #
#									  #
#          check that dhcp is up and running.  Right now we don't check   #
#          the /etc/dhcpd.conf file, since that should've been checked    #
#          when made.  This function is just for extreme paranoya.        #
#									  #
#-------------------------------------------------------------------------#

sub verify_dhcp
{
        my $errors = 0;
        my @output;
        my $cmd;
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDhcpCheck');
        unless(-f "/var/lib/dhcp/dhcpd.leases"){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I', 'IMsgDhcpMissing');
            unless(-d "var/lib/dhcp"){
                NodeUtils->runcmd("$::MKDIR -p /var/lib/dhcp");
            }
            NodeUtils->runcmd("$::TOUCH /var/lib/dhcp/dhcpd.leases");
        }
        # SuSE Change
        #$cmd = "$::SERVICE dhcpd status";
        $cmd=NodeUtils->service("dhcpd","status");

        chomp(@output = `$cmd`);
        my @line = split(" ",@output);
        if( grep /running/, @output){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDhcpOutput', @output);
        }
        else{
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDhcpOutput', @output);
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDhcpProblemRestart', @output);
            # SuSE Change
            #`$::SERVICE dhcpd start`;
            #$cmd = "$::SERVICE dhcpd status";
            $cmd=NodeUtils->service("dhcpd","start");
            `$cmd`;
            $cmd=NodeUtils->service("dhcpd","status");

            chomp(@output = `$cmd`);
            if( grep /running/, @output){
                #print "dhcp: @output\n" if $::VERBOSE;
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDhcpOutput', @output);
            }
            else{
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I', 'IMsgDhcpNotWorking');
                $errors++;
            }
        }
        return $errors;
}

#-------------------------------------------------------------------------#
#									  #
#      testvalidconsole					                  #
#									  #
#            checks for the valid Console Method attributes. This         #
#            subroutine  compares the given Console method against the    #
#            list in the /opt/csm/bin/*_console Console methods.          #
#									  #
#-------------------------------------------------------------------------#
sub testvalidconsole
{
  my ($class, $my_console) = @_;
  my @temp_console=();
  @::valid_console=();
  my $console="";
  my $tmp_console="";
  @temp_console=NodeUtils->runcmd("$::LS \/opt\/csm\/bin\/\*_console", -1);
  if ($::RUNCMD_RC != 0)
  {
    return 2; 
  }
  foreach $tmp_console (@temp_console)
  {
     (undef, undef, undef,undef, $console)=split("/",$tmp_console);
     ($console, undef)=split("_",$console);
     push(@::valid_console, $console);
  }
  if (!grep(/^$my_console$/, @::valid_console))
  {
    return 1;
  }
  else
  {
    return 0;
  }
}


#-------------------------------------------------------------------------#
#									  #
#      testvalidpowermethod                                               #
#									  #
#            checks for the valid Power Method attributes. This           #
#            subroutine  compares the given Power method against the one  #
#            in the /opt/csm/lib/lib+<given Power Method>+.so. If this    #
#            Method is not found then it checks against the               #
#            /opt/csm/bin/*_power Power Methods                           #
#									  #
#-------------------------------------------------------------------------#
sub testvalidpowermethod
{
  my ($class, $my_powermethod) = @_;
  my @temp_powermethod=();
  @::valid_powermethod=();
  my $powermethod="";
  my $tmp_powermethod="";
  #my $my_temp_powermethod="lib".$my_powermethod.".so";
  my $my_temp_powermethod="lib".$my_powermethod;
  my $exit_code=0;
  @temp_powermethod=();
  @temp_powermethod=NodeUtils->runcmd("$::LS \/opt\/csm\/lib\/\*_power\.so", -1);
  if ($::RUNCMD_RC != 0)
  {
    $exit_code=2;
  }
  else
  {
    foreach $tmp_powermethod (@temp_powermethod)
    {
      (undef, undef, undef,undef, $powermethod)=split("/",$tmp_powermethod);
      ($powermethod, undef)=split("_",$powermethod);
      push(@::valid_powermethod, $powermethod);
    }
    if (!grep(/^$my_temp_powermethod$/, @::valid_powermethod))
    {
      $exit_code=1; 
    }
    else
    {
      $exit_code=0; 
    }
  }
  if ( ($exit_code ==1) || ($exit_code ==2) )
  {
    @temp_powermethod=();
    $my_temp_powermethod=$my_powermethod;
    @temp_powermethod=NodeUtils->runcmd("$::LS \/opt\/csm\/bin\/\*_power", -1);
    if ($::RUNCMD_RC != 0)
    {
      #$exit_code=2;
    }
    else
    {
      foreach $my_temp_powermethod (@temp_powermethod)
      {
        (undef, undef, undef,undef, $powermethod)=split("/",$tmp_powermethod);
        ($powermethod, undef)=split("_",$powermethod);
        push(@::valid_powermethod, $powermethod);
      }

      if (!grep(/^$my_temp_powermethod$/, @::valid_powermethod))
      {
        $exit_code=1; 
      }
      else
      {
        $exit_code=0; 
      }
    } 
   }
   return $exit_code;
# end new code
}

#-------------------------------------------------------------------------#
#									  #
#      chkethup                                                           #
#          Ensure that the eth0 is up and running                         #
#									  #
#-------------------------------------------------------------------------#
sub chkethup
{
    my $cmd = "$::NETSTAT -rn | $::GREP eth0";
    unless(`$cmd`)
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1','EMsgEth0NotUp');
    }
  return;
}

#-------------------------------------------------------------------------#
#                                                                         #
#     getRecentVersionRpm						  #
#                                                                         #
#       returns         $recent rpm                                       #
#                                                                         #
#-------------------------------------------------------------------------#
sub getRecentVersionRpm
{
  my ($class, $path, $rpm_name) = @_;

  my @localversions=();
  my %packver;
  my @output=`ls  -1 $path/$rpm_name`;
  chomp @output;
  foreach my $rpm1 (@output)
  {
    my $VERSION=`rpm -qp --qf '%{Version}' $rpm1`;
    chomp $VERSION;
    $packver{"$VERSION"}=$rpm1;
    push (@localversions, $VERSION);
  }
  my $versionout="";
  my $i=0;
  foreach my $my_version (@localversions)
  {
    if ($i==0) {$versionout=$my_version; }
    if (InstallUtils->testVersion($my_version,">",$versionout))
    {
      $versionout=$my_version;
    }
    $i++;
  }

  return ($packver{$versionout});
}

#-------------------------------------------------------------------------#
#									  #
#      subnet       						          #
#									  #
#          Calculate the subnet that a host is on.                        #
#      INPUT:                                                             #
#	   host    : hostname or ip address                               #
#	   netmask : subnet mask in dotted decimal format (a.b.c.d)       #
#      OUTPUT:                                                            #
#	   returns the subnet in dotted decimal format (a,b,c,d)          #
#									  #
#-------------------------------------------------------------------------#
sub subnet
{
    my ($class, $host, $netmask) = @_;

    #$host = `hostname` if (!$host);
    #$netmask = '255.255.255.0' if (!$netmask);

    chomp $host;
    chomp $netmask;

    #print "INPUT:  $host $netmask\n";

    my ($hostname, $ipaddr) = NodeUtils->getHost($host);

    my ($ia,$ib,$ic,$id) = split ('\.', $ipaddr);
    my ($na,$nb,$nc,$nd) = split ('\.', $netmask);

    # Convert to integers so the bitwise and (&) works correctly.
    int $ia; int $ib; int $ic; int $id;
    int $na; int $nb; int $nc; int $nd;

    my $sa = ($ia & $na);
    my $sb = ($ib & $nb);
    my $sc = ($ic & $nc);
    my $sd = ($id & $nd);
    my $subnet = "$sa.$sb.$sc.$sd";


    #print "IPADDR = $ia.$ib.$ic.$id\n";
    #print "NETMSK = $na.$nb.$nc.$nd\n";
    #print "SUBNET = $subnet\n";

    return $subnet;
}

#-------------------------------------------------------------------------#
#									  #
#      determineMS    						          #
#									  #
#	this returns the name of the localhost from a list of node names
#	input: @DestNode
#									  #
#-------------------------------------------------------------------------#
sub determineMS
{
	my ($class, $refnodes) = @_;
	my @nodes = @$refnodes;
	my ($localnode, $ifconfig, $label, $separator, $address, @ipaddrs, %localhostnames);
	
	if($::PLTFRM eq "AIX"){
		$ifconfig = "/usr/sbin/ifconfig -a";
		# format is: "inet xxx.xxx.xxx.xxx
		$label = "inet";
		$separator = " ";
	}
	else{ #Linux
		$ifconfig = "/sbin/ifconfig -a";
		# format is: "inet addr:xxx.xxx.xxx.xxx
		$label = "inet addr";
		$separator = ":";
	}
	
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCMD', $ifconfig);
	my @ips = `$ifconfig 2>&1`;
	my $rc  = $? >> 8;
	if ($rc) {
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $ifconfig, $rc);
	}
	foreach(@ips){
		if ($_ =~ /$label$separator((\d{1,3}?\.){3}(\d){1,3})\s/o) {
			#print "found \"$1\" in \"$_\"\n";
			push @ipaddrs, $1;
		}
	}
	foreach my $addr (@ipaddrs){
		#see what it resolves to
		$localhostnames{NodeUtils->tryHost($addr)}=1;
	}
	FINDMS: foreach my $name (@nodes){
		my $rn = NodeUtils->tryHost($name);
		if($localhostnames{$rn}){
			#there should only be one ManagedNode that maps to the localhost
			$localnode = $name;
			last FINDMS;
		}
	}
	return $localnode;	
}

#-------------------------------------------------------------------------#
#									  #
#      getUpdateDir    						          #
#									  #
#	this returns the updates directory corresponding to the 
#	Install attributes that it is passed
#
#	input: (InstallOSName, InstallDistributionName, 
#		InstallDistributionVersion, InstallPkgArchitecture);
#									  #
#-------------------------------------------------------------------------#

sub getUpdateDir{
	my ($class, $os, $distributor, $version, $arch) = @_;
	return "/csminstall/$os/$distributor/$version/$arch/updates";
}

#-------------------------------------------------------------------------#
#									  #
#      getRPMSDir    						          #
#									  #
#	this returns the updates directory corresponding to the 
#	Install attributes that it is passed
#
#	input: (InstallOSName, InstallDistributionName, 
#		InstallDistributionVersion, InstallPkgArchitecture);
#									  #
#-------------------------------------------------------------------------#

sub getRPMSDir{
	my ($class, $os, $distributor, $version, $arch) = @_;
	return "/csminstall/$os/$distributor/$version/$arch/RPMS";
}

1;
